macro_file MultiCut; // Tab setting for this file is 2
/**************************Multi-Edit Macro*********************************
												MultiCut Version 1.00
										 Written by Donald K Forbes
							(Modifications for buffer viewing by Al O'Donnell
 								- fixes by Kevin R Jackson, Roy Jennings)

MultiCut		- Copies, copy/Appends, Moves or mOve/Appends a block
							 into 1 of N  hidden buffers. 26 is the default (User
							 configurable.) Put
							 Set_Global_Int('@MCMaxBuffers', <x>);
							 in your startup to set 'N'.
MultiPaste  - Copies a block from 1 of 26 hidden buffers into current
							 file.
MultiSwitch - Switch backward or forward (next or previous buffer).
MultiMenu   - Multi Cut and Paste menu.

MultiView		- Browse the cut buffers to select the one you want! Allows deleting
							buffers, and marking them as "read-only" to protect them.

These macros implement a 26 buffer cut and paste system that can be
implemented in one of two ways (or both):

	1.  A cut and paste system where you specify on which of the 26
			buffers the operation is to be performed.

	2.  A cut and paste system that uses each of the 26 buffers in a
			circular queue.

	3.  Both (might be confusing).

In the first method every time you copy or move a block to a buffer
you must specify to which buffer the copy or move is to be performed
and every time you want to paste from a buffer you must specify from
which buffer the paste is to be performed.  If you do an 'Append to
buffer' operation the current block is appended to the buffer you
specify.

In the second method every time you copy or move a block to a buffer
the block is copied or moved to one of the buffers in a circular
queue.  Each consecutive time you do a copy or move, the operation is
performed on the next buffer in the queue until buffer 26 is reached.
When buffer 26 is reached the next buffer in the queue becomes buffer
1.  If you do an 'Append to buffer' operation the contents of the
previous buffer in the queue are copied to the current buffer in the
queue and the current block is then appended to the current buffer in
the queue.  You can also switch to the previous or next buffer in the
queue with this method.

Although it may become confusing, both methods mentioned above can be
used in the same edit session and both methods use the same 26
buffers.  The 26 buffers that are used by these macros are numbered 1
through 26 (A through Z) and are separate from buffer 0 that is used
by Multi-Edit's cut and paste macros.

Key assignments:

	Ŀ
	                                                                          
	                   Multi Cut and Paste Method 1                           
	                                                                          
	Description                       Macro      Parameter     Menu Identifier
	Ĵ
	Copy block to buffer 1..26       MultiCut   /O=1        !MCPM_KEY1     
	Copy/Append block to buffer 1..26MultiCut   /A=1/O=1    !MCPM_KEY2     
	Move block to buffer 1..26       MultiCut   /M=1/O=1    !MCPM_KEY3     
	Move/Append block to buffer 1..26MultiCut   /M=1/A=1/O=1!MCPM_KEY4     
	Paste from buffer 1..26          MultiPaste /O=1        !MCPM_KEY5     
	View cut buffers                 MultiView  /O=1        !MCPM_KEY6     
	Multi cut and paste 1..26 menu   MultiMenu  /Q=0        !MCPM_KEY      
	

	Ŀ
	                                                                          
	                   Multi Cut and Paste Method 2                           
	                                                                          
	Description                       Macro      Parameter     Menu Identifier
	Ĵ
	Copy block to queue              MultiCut   /O=1/Q=1         !MCPMQ_KEY1
	Copy/Append block to queue       MultiCut   /A=1/O=1/Q=1     !MCPMQ_KEY2
	Move block to queue              MultiCut   /M=1/O=1/Q=1     !MCPMQ_KEY3
	Move/Append block to queue       MultiCut   /M=1/A=1/O=1/Q=1 !MCPMQ_KEY4
	Paste from Queue                 MultiPaste /O=1/Q=1         !MCPMQ_KEY5
	Move backward in queue           MultiSwitch/D=0             !MCPMQ_KEY6
	Move forward in queue            MultiSwitch/D=1             !MCPMQ_KEY7
	View cut buffers                 MultiView  /O=1             !MCPMQ_KEY8
	Multi cut and paste queue menu   MultiMenu  /Q=1             !MCPMQ_KEY 
	

NOTE:  1.	All keymap settings listed above require the 'Macro file'
					parameter to be set to 'MultiCut'

			 2. The '/O=1' parameters listed above causes the currenly
					marked block to be turned off immediately after the copy,
					move or paste operation; if this is not desirable, remove
					them.

The following are the keys I used for testing these macros and are
not necessarily recommended:
														          
                           Method 1Method 2 
																		  
														Ŀ
	Multi Cut/Paste 1..26 menuAltShftF1AltShftF2 Multi Cut/Paste queue menu
														
														Ŀ
				Copy to buffer 1..26AltShftF1AltShftF2 Copy to queue
														Ĵ
 copy/Append to buffer 1..26AltShftF3AltShftF4 copy/Append to queue
														Ĵ
				Move to buffer 1..26AltShftF5AltShftF6 Move to queue
														Ĵ
 mOve/append to buffer 1..26AltShftF7AltShftF8 mOve/append to queue
														Ĵ
		 Paste from buffer 1..26AltShftF9AltShftF10Paste from queue
														
																		Ŀ
																		AltShftGREY-- move backward in queue
																		
																		Ŀ
																		AltShftGREY++ move forward in queue
																		

						(C) Copyright 1992 American Cybernetics, Inc.
****************************************************************************/
#include alssrc\multicut.sh

#ifdef WINDOWS
#include src\metools.sh
#include src\dialog.sh
#define screen_length	win_CHeight
#define screen_width  win_CWidth
#define CREATE_MENU			DlgCreate(menu)
#define DELETE_MENU			DlgKill(menu)
#else
#include src\mesys.sh
#define CREATE_MENU 		menu = menu_create()
#define DELETE_MENU			menu_delete(menu)
#endif

/**************************Multi-Edit Macro*********************************

 NAME:         MultiCut

 DESCRIPTION:  Copies or moves the currently marked block to 1 of 26 hidden
							 buffers; erases or retains the contents of the hidden buffer.

 PARAMETERS:   /M  - If present, moves the block else copies the block.
							 /A  - If present, appends the block to the hidden buffer else
											 erases the contents of the hidden buffer.
							 /O= - If non-zero the currently marked block will be turned
											 off after the block operation.
							 /Q= - If non-zero use the buffers in a circular queue.
							 /X= - The starting screen column.
							 /Y= - The starting screen line.

 RETURNS:      Nothing.

						(C) Copyright 1992 American Cybernetics, Inc.
****************************************************************************/
void MultiCut (
	int move = Parse_Int('/M=', mparm_str), append = Parse_Int('/A=', mparm_str),
			off_block = Parse_Int('/O=', mparm_str), queue = Parse_Int('/Q=', mparm_str),
			x = Parse_Int('/X=', mparm_str), y = Parse_Int('/Y=', mparm_str)
	)Trans2
{
	str		OParm[12]='/O='+Str(off_block), title='Copy', select;
	int		i, menu, next_buf, cur_buf, tmp_buf,
				Lastmenu=1,	cwin, done = TRUE, count=0, OldRefresh=Refresh;

	Push_Undo;
	Refresh = FALSE;
	next_buf = Global_Int('@MCNextBuf');
	cur_buf = next_buf;
	cwin = cur_window;
	MultiSetMaxBuffs();
	/* Check if this is a MOVE operation */
	if (move)	{
		OParm=OParm+'/M=1';
		title='Move';
		Lastmenu+=2;
	}
	/* Check if this is an append operation */
	if (append)	{
		OParm=OParm+'/A=1';
		title=title+'//Append';
		Lastmenu+=1;
	}

	do {
		done = TRUE;
		switch_window(cwin);
		if (!queue) {
			CREATE_MENU;
			title=title+' block to buffer.';
			select = "Select buffer to use.";
#ifdef Windows
 			DlgAddCtrl(menu, DLG_Static, select, 1, 1, length(select), 1, STATIC_CTRL, 0, "");
 			DlgAddCtrl(menu, DLG_Integer, "", Dlg_PosOffset|length(select), 1, 3, 0, INTEGER_CTRL, 0,
				"/MIN=1/MAX="+str(Global_Int('@MCMaxBuffers')));
 			DlgSetInt(menu, INTEGER_CTRL, next_buf);
  		DlgAddCtrl(menu, DLG_PushButton, "&OK", 1, Dlg_PosOffset|1, Dlg_StanBtnWidth, 0,
				OK_CTRL, Dlgf_DefButton, OK_RET);
  		DlgAddCtrl(menu, DLG_PushButton, "&Cancel", Dlg_PosOffset|Dlg_StanBtnWidth + 1, Dlg_PosOffset, Dlg_StanBtnWidth, 0,
				CANCEL_CTRL, Dlgf_DefButton, CANCEL_RET);
  		DlgAddCtrl(menu, DLG_PushButton, "&Help", Dlg_PosOffset|Dlg_StanBtnWidth + 1, Dlg_PosOffset,
   			Dlg_StanBtnWidth, 0, HELP_CTRL, Dlgf_DefButton, HELP_RET);
			Return_Int = DlgExecute(menu, INTEGER_CTRL, title, "", "", 0);
			if (return_int!=0) {
    		next_buf = DlgGetInt(menu, INTEGER_CTRL);
				return_int = next_buf;
			}
#else
			for (i = 1; i < (Global_Int('@MCMaxBuffers')+1); i++)
				menu_Set_Item(menu,i,Str(i),'','',0, 0,0);

			Return_Int=menu;
			RM('UserIn^Topmenu /HN=1/#=' + str(Global_Int('@MCMaxBuffers')) + '/L='+title+' '+select+
				'/S='+Str(Next_buf)+'/X='+str(x)+'/Y='+str(y));
#endif
			tmp_buf = return_int;
			DELETE_MENU;
		}	else {
			count++;
			if ((Global_Int('@MCMaxBuffers')+1) <= (tmp_buf = ++next_buf)) {
				tmp_buf=1;
				next_buf = 1;
			}
			if (append)	{
				int TWin=Window_Id,ro = FALSE;

				if (switch_file((_BUFFER_NAME+Str(tmp_buf)))) {
					if (read_only) {
						ro = TRUE;
					}
				}
				if (!ro) {
					if (Switch_File(_BUFFER_NAME+Str(cur_buf))) {
	 					RM('Cut /B='+Str(tmp_buf));
 						Switch_Win_Id(TWin);
					}
				}
			}
		}

		if (switch_file((_BUFFER_NAME+Str(tmp_buf)))) {
			if (read_only) {
				done = FALSE;
				if (!queue) {
					RM("MEERROR^MessageBox /B=1/T=Read Only Buffer!/M=Select another buffer.");
				}
			}
		}
	} while (!done && (count <= Global_Int('@MCMaxBuffers')));
	switch_window(cwin);

	if (count >  Global_Int('@MCMaxBuffers')) {
		RM("MEERROR^MessageBox /B=1/T=No Free Buffer!/M=All Buffers are READ ONLY.");
	} else {
		if ((next_buf=tmp_buf) != 0) {
			RM('Cut /B='+Str(next_buf)+OParm);
			Set_Global_Int('@MCNextBuf',next_buf);
			Set_Global_Int('@MCLastmenu',Lastmenu);
		}
	}
	Refresh=OldRefresh;
	Redraw;
	Pop_Undo;
} // MultiCut

/**************************Multi-Edit Macro*********************************

 NAME:         MultiPaste

 DESCRIPTION:  Pastes the contents from 1 of 26 hidden buffres to the
							 current cursor position.

 PARAMETERS:   /O= - If non-zero the currently marked block will be turned
											 off after the block operation.
							 /Q= - If non-zero use the buffers in a circular queue.
							 /X= - The starting screen column.
							 /Y= - The starting screen line.

 RETURNS:      Nothing

						(C) Copyright 1992 American Cybernetics, Inc.
****************************************************************************/
void MultiPaste(
	int off_block = Parse_Int('/O=', mparm_str), queue = Parse_Int('/Q=', mparm_str),
			x = Parse_Int('/X=', mparm_str), y = Parse_Int('/Y=', mparm_str)
	) Trans2
{
	str		OParm[4]='/O='+Str(off_block!=0), title, select;
	int		i, menu, Cur=Global_Int('@MCNextBuf');

	Push_Undo;
	MultiSetMaxBuffs();
	if (!queue) {
		title = 'Paste block from buffer.';
		select = 'Select buffer to use:';
		CREATE_MENU;
#ifdef Windows
 		DlgAddCtrl(menu, DLG_Static, select, 1, 1, length(select), 1, STATIC_CTRL, 0, "");
 		DlgAddCtrl(menu, DLG_Integer, "", Dlg_PosOffset|length(select), 1, 3, 0, INTEGER_CTRL, 0,
			"/MIN=1/MAX="+str(Global_Int('@MCMaxBuffers')));
 		DlgSetInt(menu, INTEGER_CTRL, Cur);
  	DlgAddCtrl(menu, DLG_PushButton, "&OK", 1, Dlg_PosOffset|1, Dlg_StanBtnWidth, 0,
			OK_CTRL, Dlgf_DefButton, OK_RET);
  	DlgAddCtrl(menu, DLG_PushButton, "&Cancel", Dlg_PosOffset|Dlg_StanBtnWidth + 1, Dlg_PosOffset, Dlg_StanBtnWidth, 0,
			CANCEL_CTRL, Dlgf_DefButton, CANCEL_RET);
  	DlgAddCtrl(menu, DLG_PushButton, "&Help", Dlg_PosOffset|Dlg_StanBtnWidth + 1, Dlg_PosOffset,
   		Dlg_StanBtnWidth, 0, HELP_CTRL, Dlgf_DefButton, HELP_RET);
		Return_Int = DlgExecute(menu, INTEGER_CTRL, title, "", "", 0);
		if (return_int!=0) {
    	cur = DlgGetInt(menu, INTEGER_CTRL);
			return_int = cur;
		}
#else
		for (i = 1; i < (Global_Int('@MCMaxBuffers')+1); i++)
			Menu_Set_Item(menu,i,Str(i),'','',0, 0,0);

		Return_Int=menu;
		RM('UserIn^TopMenu /HN=1/#=' + str(Global_Int('@MCMaxBuffers')) + '/L='+title+' '+select+
				'/S='+Str(Cur)+'/X='+str(x)+'/Y='+str(y));
		Cur=Return_Int;
#endif
		DELETE_MENU;
	} else {
		if (!Cur){
			Cur=1;
		}
		Return_Int=Cur;
	}

	if (0 != (Cur=Return_Int))	{
		RM('Paste /B='+Str(Cur)+OParm);
		Set_Global_Int('@MCNextBuf',Cur);
		Set_Global_Int('@MCLastMenu',5);
	}
	Pop_Undo;
} // MultiPaste

/**************************Multi-Edit Macro*********************************

 NAME:         MultiSwitch

 DESCRIPTION:  Move the current queue pointer backward or forward.

 PARAMETERS:	/D= Direction to move.  0 to move backward.
																			1 to move forward.

 RETURNS:      Nothing

						(C) Copyright 1992 American Cybernetics, Inc.
****************************************************************************/
void MultiSwitch(
	int direction=Parse_Int('/D=', mparm_str)
	) Trans2
{
	int Cur;

	MultiSetMaxBuffs();
	if (direction) {
		Cur = 1;
	} else {
		Cur = -1;
	}

	if (0 >= (Cur=(Global_Int('@MCNextBuf')+Cur))) {
		Cur = Global_Int('@MCMaxBuffers');
	} else if ((Global_Int('@MCMaxBuffers')+1) <= Cur) {
		Cur = 1;
	}

	Make_Message('Switched to buffer '+str(Cur));
	Set_Global_Int('@MCNextBuf',Cur);
} // MultiSwitch

/**************************Multi-Edit Macro*********************************

 NAME:         MultiMenu

 DESCRIPTION:  Displays 1 of 2 menus for the multi cut and paste
							 system.

 PARAMETERS:	 /Q= - 0 Display menu for user selected system.
									 - 1 Display menu for queued system.
							 /X= - The starting screen column.
							 /O=<0,1> - Block is turned off after operation.
							 /Y= - The starting screen line.

 RETURNS:      Nothing

						(C) Copyright 1992 American Cybernetics, Inc.
****************************************************************************/
void MultiMenu(
	int off_block = Parse_Int('/O=', mparm_str), queue = Parse_Int('/Q=', mparm_str),
			x = Parse_Int('/X=', mparm_str), y = Parse_Int('/Y=', mparm_str))
{
	str MenuTail[22], Q[1]='',PasteTail[20];
	int i=1,Items=6+(2*(queue!=0)),Direction=0,
			move = 0, append = 0, direction_item = 0, view_item=0,
			menu,LastMenu=Global_Int('@MCLastMenu');

	MultiSetMaxBuffs();
	if (Items == 6) {
		MenuTail=' block to buffer 1..' + Str(Global_Int('@MCMaxBuffers'));
		PasteTail=' from buffer 1..'+Str(Global_Int('@MCMaxBuffers'));
	} else {
		MenuTail=' block to queue';
		PasteTail=' from queue';
		Q='Q';
	}
	CREATE_MENU;
#ifdef Windows
  DlgAddCtrl(menu, DLG_RadioButton, 'Copy'+MenuTail, 1, 1, RADIO_TEXT_LEN, 0, RADIO_COPY, 0, "");
  DlgAddCtrl(menu, DLG_RadioButton, 'Copy/Append'+MenuTail, 1, Dlg_PosOffset | Dlg_Units | 9,
		RADIO_TEXT_LEN, 0, RADIO_COPY_A, 0, "");
  DlgAddCtrl(menu, DLG_RadioButton, 'Move'+MenuTail, 1, Dlg_PosOffset | Dlg_Units | 9,
		RADIO_TEXT_LEN, 0, RADIO_MOVE, 0, "");
  DlgAddCtrl(menu, DLG_RadioButton, 'Move/Append'+MenuTail, 1, Dlg_PosOffset | Dlg_Units | 9,
		RADIO_TEXT_LEN, 0, RADIO_MOVE_A, 0, "");
  DlgAddCtrl(menu, DLG_RadioButton, 'Paste', 1, Dlg_PosOffset | Dlg_Units | 9,
		RADIO_TEXT_LEN, 0, RADIO_PASTE, 0, "");
	if (Items > 6) {
  	DlgAddCtrl(menu, DLG_RadioButton, 'Move Back in Queue', 1, Dlg_PosOffset | Dlg_Units | 9,
			RADIO_TEXT_LEN, 0, RADIO_BACK, 0, "");
  	DlgAddCtrl(menu, DLG_RadioButton, 'Move Forward in Queue', 1, Dlg_PosOffset | Dlg_Units | 9,
			RADIO_TEXT_LEN, 0, RADIO_FORWARD, 0, "");
	}
 	DlgAddCtrl(menu, DLG_RadioButton, 'View Cut Buffers', 1, Dlg_PosOffset | Dlg_Units | 9,
		RADIO_TEXT_LEN, 0, RADIO_VIEW, 0, "");
	for (x=RADIO_START;x<=(RADIO_START+(Items-1));x++) {
  	DlgSetInt(menu, x, 0);
	}
  DlgSetInt(menu, (LastMenu-1)+RADIO_START, 1);
  DlgAddCtrl(menu, DLG_PushButton, "&OK", 1, Dlg_PosOffset|1, Dlg_StanBtnWidth, 0,
		OK_CTRL, Dlgf_DefButton, OK_RET);
  DlgAddCtrl(menu, DLG_PushButton, "&Cancel", Dlg_PosOffset|Dlg_StanBtnWidth + 1, Dlg_PosOffset, Dlg_StanBtnWidth, 0,
		CANCEL_CTRL, Dlgf_DefButton, CANCEL_RET);
  DlgAddCtrl(menu, DLG_PushButton, "&Help", Dlg_PosOffset|Dlg_StanBtnWidth + 1, Dlg_PosOffset,
   	Dlg_StanBtnWidth, 0, HELP_CTRL, Dlgf_DefButton, HELP_RET);
	Return_Int = DlgExecute(menu, OK_CTRL, "MULTI CUT AND PASTE", "", "", 0);
	if (return_int) {
		direction_item = 0;
		for (x=RADIO_START;(x<=(RADIO_START+(Items-1))) && (direction_item == 0);x++) {
			DlgGetInt(menu, x);
			if (DlgGetInt(menu, x)) {
				direction_item = (x-RADIO_START)+1;
			}
		}
		if (direction_item>=6) {
			if (direction_item == 6) {
				direction_item += 2;
			} else {
				direction_item -= 1;
			}
		}
		view_item = (RADIO_VIEW-RADIO_START)+1;
		return_int = direction_item;
	}
#else
	Menu_Set_Item(menu,i,'Copy'+MenuTail,Global_Str('!MCPM'+Q+'_KEY'+Str(i)), '',0,0,0); i++;
	Menu_Set_Item(menu,i,'copy/Append'+MenuTail,Global_Str('!MCPM'+Q+'_KEY'+Str(i)), '',0,0,0); i++;
	Menu_Set_Item(menu,i,'Move'+MenuTail,Global_Str('!MCPM'+Q+'_KEY'+Str(i)), '',0,0,0); i++;
	Menu_Set_Item(menu,i,'mOve/append'+MenuTail,Global_Str('!MCPM'+Q+'_KEY'+Str(i)), '',0,0,0); i++;
	Menu_Set_Item(menu,i,'Paste'+PasteTail,Global_Str('!MCPM'+Q+'_KEY'+Str(i)), '',0,0,0); i++;
	if (Items > 6)	{
		Menu_Set_Item(menu,i,'- move backward',Global_Str('!MCPM'+Q+'_KEY'+Str(i)), '',0,0,0); i++;
		Menu_Set_Item(menu,i,'+ move forward',Global_Str('!MCPM'+Q+'_KEY'+Str(i)), '',0,0,0); i++;
	}
	Menu_Set_Item(menu,i,'View Cut Buffers',Global_Str('!MCPM'+Q+'_KEY'+Str(i)), '',0,0,0); i++;

	Return_Int=menu;
	RM('UserIn^SubMenu /HN=1/#='+Str(Items)+'/L=MULTI CUT AND PASTE/S='+
			Str(LastMenu)+'/X='+str(x)+'/Y='+str(y));
	if ((Items == 6) && (Return_Int >= 6)) {
		Return_int += 2;
	}
	direction_item = return_int-1;
	view_item=Items;
	Kill_Box;
#endif
	switch (Return_Int)	{
		case 4 :
			move = 1;
		case 2 :
			append = 1;
			MultiCut(move, append, off_block, queue, x, y);
			break;

		case 3 :
			move = 1;
		case 1 :
			MultiCut(move, append, off_block, queue, x, y);
			break;

		case 5 :
			MultiPaste(off_block, queue, x, y);
			break;

		case 7 :
			Direction=1;
		case 6 :
			MultiSwitch(direction);
			Set_Global_Int('@MCLastMenu',direction_item+1);
			break;
		case 8:
			MultiView(off_block);
			Set_Global_Int('@MCLastMenu',view_item);
			break;
	}
	DELETE_MENU;
} // MultMenu

/**************************Multi-Edit Macro*********************************

 NAME:         MultiView

 DESCRIPTION:  Displays the current edit buffer, and allows cycling through each.

 PARAMETERS:	 /O=<0,1> - Block is turned off after operation.

 RETURNS:      Nothing

						(C) Copyright 1992 American Cybernetics, Inc.
****************************************************************************/
void MultiView (int off_block = parse_int('/O=', mparm_str))
{
	int Cur, Initial, Prev, found = FALSE, s_length = screen_length, s_width = screen_width;
	int wattr, size, direction=1, line, wide, no_buff = 0, queue=1;
	int cwin = cur_window, x=0, y=0;
	int o_refresh = Refresh;
	str name, t_name;
	int done = FALSE;
	int o_block_stat, f_changed;

	refresh = FALSE;
	Prev=Initial=Global_Int('@MCNextBuf');
	if (Initial == 0) {
		RM("MEERROR^MessageBox /B=1/T=No Edit Buffer/M=There are no buffers! ");
	} else {
		do {
			wattr = window_attr;
			f_changed = file_changed;
			o_block_stat = 0;
			Cur=Global_Int('@MCNextBuf');
			if (Cur == 0) {
				Cur++;
			}
			name = _BUFFER_NAME+Str(Cur);
			t_name = _BUFFER_NAME+Str(Cur);
			if (Switch_File(name)) {
				f_changed = FALSE;
				no_buff = 0;
 				wattr = window_attr;
				window_attr = 00;
				if (block_stat) {
					o_block_stat = block_stat;
					if (block_line1 < block_line2) {
						size = (block_line2 - block_line1) + 2;
						line = block_line1;
					} else {
						size = (block_line1 - block_line2) + 2;
						line = block_line2;
					}
					if (size > (s_length/2)) {
						size = s_length/2;
					}
					goto_line(line);
					RM('TOPWIN');
					goto_line(line);
				} else {
					eof;
					DOWN;
					size = 2;
				}
				wide = s_width - 6;
			}	else {
				size = 1;
				no_buff = 1;
				wide = 1;
			}
			MultiWindow("Browsing "+t_name, "", 3, 5, wide, size, s_width, s_length, no_buff);
			if (o_block_stat) {
				block_stat = o_block_stat;
			}
			window_attr = wattr;
			file_changed = f_changed;
			switch (return_int) {
				case 0:	/* Cancel */
					done = TRUE;
					break;
				case  1:	/* Ok */
					Switch_window(cwin);
					MultiPaste(off_block, queue, x, y);
					done = TRUE;
					break;
				case 3: /* switch direction! */
					found = TRUE;
					Prev = Cur;
					direction = 1;
					MultiSwitch(direction);
					break;
				case 4: /* switch backward! */
					found = TRUE;
					Prev = Cur;
					direction = 0;
					MultiSwitch(direction);
					break;
				case 5:	/* DELETE the window! */
					delete_window;
					found = FALSE;
					Prev = Cur;
					MultiSwitch(direction);
					break;
				case 6: /* toggle the buffers "read_only" flag */
					Read_Only = !Read_Only;
					break;
				default:
					done = TRUE;
			}
		} while (!done);
	}
	Switch_window(cwin);
	refresh = o_refresh;
	redraw;
}

/*******************************************************************************
                                MULTI_EDIT MACRO

Name: MultiWindow

Description: Creates an editable window for inputing multiple lines of text.
      Uses the current window.


Parameters:   /T=str        Title string.
              /H=str        Help string.
							/X=nn         X position
              /Y=nn         Y position
              /W=nn         Width
              /L=nn         Length
							/MW=nn				Maximum width
							/MW=nn				Maximum length
							/NEX=nn				Buffer present

               (C) Copyright 1991 by American Cybernetics, Inc.
*******************************************************************************/
void MultiWindow(
	str title = parse_str("/T=", mparm_str), help_str = parse_str("/H=", mparm_str),
	int x = parse_int('/X=', mparm_str), y = parse_int('/Y=', mparm_str),
			w = parse_int('/W=', mparm_str), l = parse_int('/L=', mparm_str),
			s_width = parse_int('/MW=', mparm_str), s_length = parse_int('/ML=', mparm_str),
			no_buff = parse_int('/NEX=', mparm_str)) TRANS2 NO_BREAK
{
  int  jx, ev_count, T_Mode, old_refresh = refresh;
  str  event_str[20];
  int accept = 1, adjust = 0, menu;

	// Initialize fields.
	refresh = FALSE;
  Set_Global_Int('MENU_LEVEL', Global_Int('MENU_LEVEL') + 1);
  T_Mode = Mode;
  Mode = Edit;
  if (x == 0) x = 10;
  if (y == 0) y = 3;
  if (w == 0) w = 40;
  if (l == 0) l = 10;
  if (w > (s_width - 3))	w=s_width - 3;
  if (l > (s_length - 7))	l=s_length - 7;

	// Set the events.
	ev_count = 1;
	CREATE_MENU;
#ifdef Windows
  DlgSetPos(menu, x*DLG_Units_Per_Row, y*DLG_Units_Per_line);
	if (!no_buff) {
  	DlgAddCtrl(menu, DLG_PushButton, "&OK", 1, Dlg_PosOffset,	Dlg_StanBtnWidth, 0, OK_CTRL,
			Dlgf_DefButton, OK_RET);
	}
  DlgAddCtrl(menu, DLG_PushButton, "&Cancel", Dlg_PosOffset|Dlg_StanBtnWidth + 1, 1, Dlg_StanBtnWidth, 0,
		CANCEL_CTRL, Dlgf_DefButton, CANCEL_RET);
  DlgAddCtrl(menu, DLG_PushButton, "&Prev", Dlg_PosOffset | Dlg_StanBtnWidth + 1, Dlg_PosOffset,
   	Dlg_StanBtnWidth, 0, PREV_CTRL, Dlgf_DefButton, PREV_RET);
  DlgAddCtrl(menu, DLG_PushButton, "&Next", Dlg_PosOffset | Dlg_StanBtnWidth + 1, Dlg_PosOffset,
   	Dlg_StanBtnWidth, 0, NEXT_CTRL, Dlgf_DefButton, NEXT_RET);
	if (!no_buff) {
  	DlgAddCtrl(menu, DLG_PushButton, "&Delete", Dlg_PosOffset | Dlg_StanBtnWidth + 1, Dlg_PosOffset,
   		Dlg_StanBtnWidth, 0, DEL_CTRL, Dlgf_DefButton, DEL_RET);
  	DlgAddCtrl(menu, DLG_PushButton, "&Read Only", Dlg_PosOffset | Dlg_StanBtnWidth + 1, Dlg_PosOffset,
   		Dlg_StanBtnWidth+2, 0, RO_CTRL, Dlgf_DefButton, RO_RET);
			adjust = Dlg_StanBtnWidth + 3;
	} else {
		adjust = Dlg_StanBtnWidth+1;
	}
  DlgAddCtrl(menu, DLG_PushButton, "&Help", Dlg_PosOffset | adjust, Dlg_PosOffset,
   	Dlg_StanBtnWidth, 0, HELP_CTRL, Dlgf_DefButton, HELP_RET);
#else
	if (!no_buff) {
  	menu_set_item(menu, ++ev_count,'OK', '', '/KC=<F10>/K2=68'+OK_RET, 11, 0, 0);
	}
  menu_set_item(menu, ++ev_count,'Cancel','','/KC=<ESC>/K1=27/K2=1'+CANCEL_RET, 11, 0, 0);
  menu_set_item(menu, ++ev_count,'-','','/KC=<F2>/K2=60'+PREV_RET, 11, 0, 0);
  menu_set_item(menu, ++ev_count,'+','','/KC=<F3>/K2=61'+NEXT_RET, 11, 0, 0);
	if (!no_buff) {
	  menu_set_item(menu, ++ev_count,'Delete','','/KC=<F5>/K2=63'+DEL_RET, 11, 0, 0);
	  menu_set_item(menu, ++ev_count,'Read Only','','/KC=<F6>/K2=64'+RO_RET, 11, 0, 0);
	}
  menu_set_item(menu, ++ev_count,'Help', '','/KC=<F1>/K2=59'+HELP_RET, 11, 0, 0);

	// Center the buttons.
  {
    int tw, xw = 0, jx, x1 = 0;
    for (jx = 2; jx <= ev_count; jx++) {
      tw = length(parse_str('/KC=', menu_item_str(menu,jx,3))) +
              length(menu_item_str(menu, jx, 1));
      menu_set_int(menu, jx, 3, tw);
      xw += tw + 2;
    }
    xw -= 2;
    x1 = (w / 2) - (xw / 2) + 1;
    if(x1 < 1)
      x1 = 1;
    for (jx = 2; jx <= ev_count; jx++) {
      menu_set_str(menu, jx, 3, menu_item_str(menu, jx, 3) + '/C=' + str(x1) + '/L=' + str(l + 3));
      x1 += menu_item_int(menu, jx, 3) + 2;
    }
  }
#endif


	// Create the dialog box.
	right_margin = 2048;
#ifdef Windows
	if (no_buff)  {
  	DlgAddCtrl(menu, DLG_Static, "No Buffer!", 1, Dlg_PosOffset|2, 6*Dlg_StanBtnWidth, 1,
			STATIC_CTRL, DLGF_ss_Center, "");
		accept = 4;
	} else {
		if (read_only) {
  		DlgAddCtrl(menu, DLG_Static, "Read Only", 1, Dlg_PosOffset|2, 15, 1, STATIC_CTRL, DLGF_ss_Left, "");
		} else {
  		DlgAddCtrl(menu, DLG_Static, "Edit", 1, Dlg_PosOffset|2, 4, 1, STATIC_CTRL, DLGF_ss_Left, "");
		}
  	DlgAddCtrl(menu, DLG_ViewTextBox, "", 1, Dlg_PosOffset|1, w, (l-(l/2)+1), TEXT_CTRL,
			0, "/WIN=" + str(cur_window));
		accept = 0;
	}
	return_int = DlgExecute(menu, CANCEL_CTRL, title, "", "", 0);
#else
	if (read_only) {
		return_str = "Read only";
	} else {
  	return_str = "Edit";
	}
	if (no_buff)  {
  		menu_set_item(menu, 1,"No buffer!","", '/L=2/W=10/C=15/HT=1', 10, 0, 0);
			accept = 4;
	} else {
  		menu_set_item(menu, 1, return_str,"",
        	'/PB=1/QK=1/C=1/L=1/W=' + str(w) + '/HT=' + str(l) + '/WIN=' + str(cur_window),
        	16, 0, 0);
			accept = 0;
	}
  return_int = menu;
  RM('USERIN^DATA_IN /HN=1/H=' + help_str + '/PRE=' + event_str +
        '/#=' + str(ev_count) + '/T=' + title + '/S=1' +
      '/X=' + str(x) + '/Y=' + str(y) +  '/A='+str(accept));
#endif


EXIT:
	Set_Global_Int('MENU_LEVEL', Global_Int('MENU_LEVEL') - 1);
  Mode = T_Mode;
  refresh = old_refresh;
	DELETE_MENU;
}

void MultiSetMaxBuffs()
{
	if ((Global_Int('@MCMaxBuffers')< 1) || (Global_Int('@MCMaxBuffers') > MAX_BUFFERS)) {
		Set_Global_Int('@MCMaxBuffers', MAX_BUFFERS);
	}
}
